@@ -92,7 +92,7 @@
 //! stress testing at least STRESS_ITERATIONS and STRESS_THREADS has to be incresed significantly.
 //! Try 'STRESS_ITERATIONS=10000 STRESS_RANGE=10000 STRESS_THREADS=10000' for some harder test.
 #![allow(clippy::type_complexity)]
-use std::sync::atomic::Ordering;
+use std::sync::atomic::{AtomicU32, Ordering};
 use std::collections::HashSet;
 use std::pin::Pin;
 
@@ -110,7 +110,7 @@ use crate::bucket::Bucket;
 pub use crate::bucket::Bucketize;
 
 mod locking_method;
-use crate::locking_method::*;
+pub use crate::locking_method::*;
 
 /// CacheDb implements the concurrent (bucketed) Key/Value store.  Keys must implement
 /// 'Bucketize' which has more lax requirments than a full hash implmementation.  'N' is the
@@ -123,7 +123,8 @@ pub struct CacheDb<K, V, const N: usize>
 where
     K: KeyTraits,
 {
-    buckets: [Bucket<K, V>; N],
+    buckets:      [Bucket<K, V>; N],
+    lru_disabled: AtomicU32,
 }
 
 impl<K, V, const N: usize> CacheDb<K, V, N>
@@ -133,7 +134,8 @@ where
     /// Create a new CacheDb
     pub fn new() -> CacheDb<K, V, N> {
         CacheDb {
-            buckets: [(); N].map(|()| Bucket::new()),
+            buckets:      [(); N].map(|()| Bucket::new()),
+            lru_disabled: AtomicU32::new(0),
         }
     }
 
@@ -211,6 +213,34 @@ where
         }
     }
 
+    /// Tries to insert an entry with the given constructor.  Returns Ok(true) when the
+    /// constructor was called, Ok(false) when and item is already present under the given key
+    /// or some Err() in case the constructor failed.
+    pub fn insert<'a, F>(&'a self, key: &K, ctor: F) -> DynResult<bool>
+    where
+        F: FnOnce(&K) -> DynResult<V>,
+    {
+        match self.query_or_insert_entry(key) {
+            Ok(_) => Ok(false),
+            Err((bucket, entry_ptr, mut map_lock)) => {
+                if self.lru_disabled.load(Ordering::Relaxed) == 0 {
+                    bucket.maybe_evict(&mut map_lock);
+                }
+
+                // need write lock for the ctor, before releasing the map to avoid a race.
+                let mut wguard = unsafe { LockingMethod::write(&Blocking, &(*entry_ptr).value)? };
+
+                // release the map_lock, we dont need it anymore
+                drop(map_lock);
+
+                // but we have wguard here which allows us to constuct the inner guts
+                *wguard = Some(ctor(key)?);
+
+                Ok(true)
+            }
+        }
+    }
+
     // TODO: The ctor function may become double nested Fn() -> Result(Fn() -> Result(Value)) The
     //       outer can acquire resouces while the cachedb is (temporary) unlocked and returns the
     //       real ctor then.
@@ -232,10 +262,13 @@ where
                 guard: unsafe { LockingMethod::read(&method, &(*entry_ptr).value)? },
             }),
             Err((bucket, entry_ptr, mut map_lock)) => {
-                bucket.maybe_evict(&mut map_lock);
+                if self.lru_disabled.load(Ordering::Relaxed) == 0 {
+                    bucket.maybe_evict(&mut map_lock);
+                }
 
                 // need write lock for the ctor, before releasing the map to avoid a race.
-                let mut wguard = unsafe { LockingMethod::write(&method, &(*entry_ptr).value)? };
+                let mut wguard =
+                    unsafe { LockingMethod::write(&Blocking, &(*entry_ptr).value).unwrap() };
 
                 // release the map_lock, we dont need it anymore
                 drop(map_lock);
@@ -271,10 +304,13 @@ where
                 guard: unsafe { LockingMethod::write(&method, &(*entry_ptr).value)? },
             }),
             Err((bucket, entry_ptr, mut map_lock)) => {
-                bucket.maybe_evict(&mut map_lock);
+                if self.lru_disabled.load(Ordering::Relaxed) == 0 {
+                    bucket.maybe_evict(&mut map_lock);
+                }
 
                 // need write lock for the ctor, before releasing the map to avoid a race.
-                let mut wguard = unsafe { LockingMethod::write(&method, &(*entry_ptr).value)? };
+                let mut wguard =
+                    unsafe { LockingMethod::write(&Blocking, &(*entry_ptr).value).unwrap() };
 
                 // release the map_lock, we dont need it anymore
                 drop(map_lock);
@@ -292,6 +328,30 @@ where
         }
     }
 
+    /// Disable the LRU eviction. Can be called multiple times, every call should be paired
+    /// with a 'enable_lru()' call to reenable the LRU finally. Failing to do so may keep the
+    /// CacheDb filling up forever. However this might be intentional to disable the LRU
+    /// expiration entirely.
+    pub fn disable_lru_eviction(&self) -> &Self {
+        self.lru_disabled.fetch_add(1, Ordering::Relaxed);
+        self
+    }
+
+    /// Re-Enables the LRU eviction after it was disabled. every call must be preceeded by a call to
+    /// 'disable_lru()'. Calling it without an matching 'disable_lru()' will panic with an integer underflow.
+    pub fn enable_lru_eviction(&self) -> &Self {
+        self.lru_disabled.fetch_sub(1, Ordering::Relaxed);
+        self
+    }
+
+    /// Checks if the CacheDb has the given key stored. Note that this will have a race
+    /// condition when other threads access the CacheDb at the same time but may make sense
+    /// when lru_eviction is disabled and it can be ensure that no other thread inserts the
+    /// key.
+    pub fn contains_key(&self, key: &K) -> bool {
+        self.buckets[key.bucket::<N>()].lock_map().contains(key)
+    }
+
     /// The 'cache_target' will only recalculated after this many inserts. Should be in the
     /// lower hundreds.
     pub fn config_target_cooldown(&self, target_cooldown: u32) -> &Self {
@@ -369,13 +429,18 @@ where
 
     /// Evicts up to number entries. The implementation is pretty simple trying to evict number/N from
     /// each bucket. Thus when the distribution is not optimal fewer elements will be removed.
-    /// Returns the number of items that got evicted
+    /// Will not remove any entries when the lru eviction is disabled.
+    /// Returns the number of items that got evicted.
     pub fn evict(&self, number: usize) -> usize {
-        let mut evicted = number;
-        for bucket in &self.buckets {
-            evicted -= bucket.evict(number / N, &mut bucket.lock_map());
+        if self.lru_disabled.load(Ordering::Relaxed) == 0 {
+            let mut evicted = number;
+            for bucket in &self.buckets {
+                evicted -= bucket.evict(number / N, &mut bucket.lock_map());
+            }
+            evicted
+        } else {
+            0
         }
-        evicted
     }
 }
 
@@ -552,6 +617,22 @@ mod test {
         );
     }
 
+    #[test]
+    fn insert_unit() {
+        init();
+        let cdb = CacheDb::<String, (), 16>::new();
+
+        assert!(cdb.insert(&"foo".to_string(), |_| Ok(())).is_ok());
+        assert_eq!(*cdb.get(Blocking, &"foo".to_string()).unwrap(), ());
+
+        assert!(cdb.insert(&"bar".to_string(), |_| Ok(())).is_ok());
+        assert_eq!(*cdb.get(Blocking, &"bar".to_string()).unwrap(), ());
+
+        assert_eq!(cdb.contains_key(&"foo".to_string()), true);
+        assert_eq!(cdb.contains_key(&"bar".to_string()), true);
+        assert_eq!(cdb.contains_key(&"baz".to_string()), false);
+    }
+
     #[test]
     fn trylocks() {
         init();
